<?php

    class Run_SQL_Query_Action extends Reactor_Action
    {
        protected $tpl = 'run_sql_query.php';
        protected $name = 'Run a SQL query';
		
		public static $registration = array(
			ACTION_NAME => 'Run a SQL query',
			ACTION_CATEGORY => 'Network',
			ACTION_DESCRIPTION => 'Run a SQL query against a DSN address with credentials.',
			ACTION_AUTHOR => 'Jeremy Yunis, Nagios Enterprises, LLC',
			ACTION_VERSION => '1.0',
			ACTION_DATE => '01/22/2013', //Last version update
			ACTION_AUTHLEVEL => 0,  // 0 - everyone | 1 - Admin only
			ACTION_TYPE => 'core', //core | user. core modules can't be deleted
			ACTION_META => array(),	//Misc information	
		); 				
		
        protected $fields = array(
            'dsn' => array(
                'label' => 'dsn',
                'validate' => 'trim|required|callback_check_dsn'
            ),
            'query' => array(
                'label' => 'sql statement',
                'validate' => 'trim|required'
            ),
        );
		
        public function get_label()
        {
            if($params = $this->get_params()) {
			
				$dsn=$params['dsn'];
				$query=substr($params['query'],0,20);
				if(strlen($query)>=20)
					$query.=" ...";
					
				//$dsnparts=split(":",$dsn);
				//$dbtype=$dsnparts[0];
				/*
				$part2=$dsnparts[2];
				$dsnparts2=split("@",$part2);
				$part3=$dsnparts2[1];
				$dsnparts3=split("/",$part3);
				$host=$dsnparts3[0];
				$db=$dsnparts3[1];
				*/
				
				//$db="DB";
				//$dbtype="DBTYPE";
				//$host="HOST";
				
                //return sprintf('Run SQL query \'%s\' on \'%s\' %s database on host \'%s\'', $query,$db,$dbtype,$host);
				return sprintf('Run SQL query \'%s\'', $query);
                }
            return parent::get_label();
        }		

        public function check_dsn($str)
        {
            if(!preg_match('/^(\w+)\:\/\/(\w+)\:(.+)\@([0-9A-Z.-]*)\/(\w+)/i', trim($str))) {
                $this->ci->form_validation->set_message('check_dsn', 'The DSN string is invalid.');      
                return FALSE;
            }
            return TRUE;
        }

        public function run($context)
        {
            $this->set_context($context);
            $dsn = $this->get_param('dsn');
            $query = $this->get_param('query');
            $this->log('info', "dsn: ". $dsn);
            $this->log('info', "query: ". $query);
            require_once "MDB2.php";
            $db =& MDB2::connect($dsn);
            if(PEAR::isError($db)) {
                $this->set_result(1);
                $this->log('error', 'Failed to connect to data source: '.$db->getMessage());
                return $context;
            }

            $affected = $db->exec($query);
            if(PEAR::isError($affected)) {
                $this->set_result(1);
                $this->log('error', 'Error performing query: '.$affected->getMessage());
                return $context;
            }
            $this->log('info', 'Query affected '.$affected.' row(s)');
            $this->set_result(0);
            return $context;
        }
    }
